import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:bubble/bubble.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:localstorage/localstorage.dart';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_comment_model.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/service/http_service.dart';
import 'package:plugin_codelab_new/widgets/post.dart';
import 'package:timeago/timeago.dart' as timeago;

class CommentsPage extends StatefulWidget {
  const CommentsPage({
    Key key,
    @required this.post,
  }) : super(key: key);

  final FeedPost post;
  @override
  _CommentsPageState createState() => _CommentsPageState();
}

class _CommentsPageState extends State<CommentsPage> {
  FocusNode focusNode;
  FeedCubit feedCubit;

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);

    focusNode = FocusNode();
    focusNode.requestFocus();
    super.initState();
  }

  final storage = new LocalStorage('my_data');

  List<PostCommentModel> commentsList = [];
  Future<List<PostCommentModel>> getCommentsList() async {
    commentsList = await feedCubit.feedRepository
        .fetchPostComments(this.widget.post.postData.id);
    return commentsList;
  }

  @override
  void dispose() {
    focusNode.dispose();
    storage.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(390, 788),
      allowFontScaling: false,
      builder: () => MaterialApp(
        home: Scaffold(
          appBar: AppBar(
            backgroundColor: Color(0xFF171725),
            leading: IconButton(
              icon: Icon(
                Icons.arrow_back,
                color: Colors.white,
              ),
              onPressed: () {
                Navigator.pop(context);
              },
            ),
          ),
          body: _buildContent(),
        ),
      ),
      // },
      // ),
    );
  }

  _buildContent() {
    final commentController = TextEditingController();
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        Expanded(
          child: Container(
            child: ListView(
              cacheExtent: 999,
              children: <Widget>[
                this.widget.post,
                _buildListView(),
              ],
            ),
            decoration: BoxDecoration(
              border: Border.all(color: Colors.grey),
            ),
          ),
        ),
        _buildTextField(commentController),
      ],
    );
  }

  _buildTextField(TextEditingController commentController) {
    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(),
      ),
      child: Row(
        children: [
          Container(
            width: ScreenUtil().setWidth(320),
            child: Padding(
              padding: EdgeInsets.only(
                left: ScreenUtil().setWidth(60),
              ),
              // top: ScreenUtil().setWidth(9)),
              child: Container(
                decoration: BoxDecoration(
                    color: Color(0xFFF5F5F5),
                    border: Border.all(
                      color: Color(0xFFF5F5F5),
                    ),
                    borderRadius: BorderRadius.all(Radius.circular(30))),
                child: Padding(
                  padding: EdgeInsets.only(
                    left: ScreenUtil().setWidth(5),
                  ),
                  child: new TextField(
                    decoration: InputDecoration(
                      labelText: "Type your comment...",
                      labelStyle: TextStyle(
                        fontSize: ScreenUtil().setWidth(20),
                        color: Color(0xFF98A9BC),
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ),
          Expanded(
              child: TextButton(
                  onPressed: () {
                    HttpService("jhgvf").uploadComment(
                      userId: 5,
                      //  storage.getItem("agentId"),
                      postId: this.widget.post.postData.id,
                      comment: commentController.text,
                    );
                    focusNode.unfocus();
                    setState(() {});
                  },
                  child: ClipOval(
                      child: Material(
                    color: Color(0xFF3DD598), // button color
                    child: InkWell(
                      splashColor: Color(0xFFFCA64F), // inkwell color
                      child: SizedBox(
                        width: ScreenUtil().setWidth(40),
                        height: ScreenUtil().setHeight(30),
                        child: Icon(
                          Icons.send,
                          color: Colors.white,
                          size: ScreenUtil().setWidth(30),
                        ),
                      ),
                      // ),
                    ),
                  )))),
          //  )
        ],
      ),
    );
  }

  _buildListView() {
    double pixelRatio = MediaQuery.of(context).devicePixelRatio;
    double px = 1 / pixelRatio;
    BubbleStyle styleSomebody = BubbleStyle(
      nip: BubbleNip.leftTop,
      elevation: 1 * px,
      margin: BubbleEdges.only(
        top: ScreenUtil().setWidth(8),
        right: ScreenUtil().setWidth(10),
        left: ScreenUtil().setWidth(50),
      ),
      alignment: Alignment.topLeft,
    );
    BubbleStyle styleImageSomebody = BubbleStyle(
      color: Colors.white.withOpacity(0),
      elevation: 0 * px,
      margin: BubbleEdges.only(
        top: ScreenUtil().setWidth(8),
      ),
      alignment: Alignment.topLeft,
    );

    return LayoutBuilder(
      builder: (_, BoxConstraints constraints) {
        return FutureBuilder(
          future: getCommentsList(),
          builder: (context, projectSnap) {
            if (projectSnap.connectionState == ConnectionState.waiting ||
                projectSnap.connectionState == ConnectionState.none) {
              return Text("");
            }
            return ListView.separated(
              separatorBuilder: (BuildContext context, int index) =>
                  const Text(""),
              shrinkWrap: true,
              physics: ClampingScrollPhysics(),
              itemCount: commentsList.length,
              itemBuilder: (BuildContext context, int index) {
                return LayoutBuilder(
                  builder: (_, BoxConstraints constraints) {
                    final date2 = DateTime.now();
                    final difference = date2
                        .difference(commentsList[index].createdAt)
                        .inMinutes;
                    final fifteenAgo = new DateTime.now()
                        .subtract(new Duration(minutes: difference));
                    UserModel userData = commentsList[index].user;

                    return Column(
                      children: [
                        Stack(
                          children: [
                            Bubble(
                              style: styleImageSomebody,
                              child: ClipOval(
                                child: Image.network(
                                  userData.profileImg,
                                  width: ScreenUtil().setWidth(35),
                                  height: ScreenUtil().setHeight(40),
                                  fit: BoxFit.cover,
                                ),
                              ),
                            ),
                            Bubble(
                              style: styleSomebody,
                              color: Color(0xffE8ECEF),
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  Row(children: [
                                    Text(userData.firstName.toString() +
                                        " " +
                                        userData.lastName.toString()),
                                    Text(
                                      " -" +
                                          timeago.format(fifteenAgo,
                                              locale: 'en_short'),
                                      style:
                                          TextStyle(color: Color(0xff98A9BC)),
                                    )
                                  ]),
                                  Text(userData.about,
                                      style:
                                          TextStyle(color: Color(0xff98A9BC))),
                                  Text(""),
                                  Text(
                                    commentsList[index].comment,
                                    style: TextStyle(
                                        fontSize: ScreenUtil().setSp(16)),
                                  ),
                                ],
                              ),
                              // ],
                              // ),
                            ),
                          ],
                        ),
                      ],
                    );
                  },
                );
              },
            );
          },
        );
      },
    );
  }
}
