import 'package:flutter/material.dart';
import 'package:plugin_codelab_new/pages/home_page.dart';
import 'package:plugin_codelab_new/repo/feed_repository.dart';
import 'package:plugin_codelab_new/service/http_service.dart';
import 'bloc/feed_cubit.dart';
import 'pages/chat/providers/messages_provider.dart';
import 'pages/login_page.dart';
import 'pages/share_post_page.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:provider/provider.dart';

class PluginCodelabNew extends StatelessWidget {
  const PluginCodelabNew({
    Key key,
    @required this.token,
    @required this.firstName,
    @required this.lastName,
    @required this.about,
    @required this.profilImg,
  }) : super(key: key);

  final String token;
  final String firstName;
  final String lastName;
  final String about;
  final String profilImg;

  @override
  Widget build(BuildContext context) {
    return init();
  }

  Widget init() {
    final httpService = HttpService("gogog");
    final feedRepository = FeedRepository(httpService);
    final feedCubit = FeedCubit(feedRepository: feedRepository);
    runApp(
      MultiRepositoryProvider(
        providers: [
          RepositoryProvider<FeedRepository>(
            create: (context) => FeedRepository(httpService),
          )
        ],
        child: MultiBlocProvider(providers: [
          BlocProvider<FeedCubit>(
            create: (context) {
              return feedCubit;
            },
          ),
        ], child: app()),
      ),
    );
    return app();
  }

  Widget app() {
    return ChangeNotifierProvider.value(
      value: MessagesProvider(),
      child: MaterialApp(
        title: 'Flutter Socket IO Chat',
        debugShowCheckedModeBanner: false,
        theme: ThemeData(
          primarySwatch: Colors.teal,
          accentColor: Colors.pink.withOpacity(.8),
          textTheme: TextTheme(
            title: TextStyle(color: Colors.white),
          ),
        ),
        home: HomePage(),
        routes: <String, WidgetBuilder>{
          '/login': (BuildContext context) => new LoginPage(),
          '/homePage': (BuildContext context) => new HomePage(),
          '/sharePostPage': (BuildContext context) => new SharePostPage(),
        },
      ),
    );
  }
}
